@@ -7,11 +7,6 @@ class Puppet::Resource::Catalog::Puppetdb < Puppet::Indirector::REST
   include Puppet::Util::Puppetdb
   include Puppet::Util::Puppetdb::CommandNames
 
-  # Run initial checks
-  def initialize
-    Puppet::Util::Puppetdb::GlobalCheck.run
-  end
-
   def save(request)
     profile "catalog#save" do
       catalog = munge_catalog(request.instance, extract_extra_request_data(request))
