@@ -72,7 +72,8 @@
                  [prismatic/schema "0.2.1"]
                  [org.clojure/tools.macro "0.1.5"]
                  [com.novemberain/pantomime "2.1.0"]
-                 [fast-zip-visit "1.0.2"]]
+                 [fast-zip-visit "1.0.2"]
+                 [robert/hooke "1.3.0"]]
 
   :jvm-opts ["-XX:MaxPermSize=128M"]
 
